@@ -3,7 +3,6 @@
 
 
 import sys
-from typing import List
 
 usage = 'source_files_or_directories destination_path'
 help_text = '''\
@@ -121,15 +120,17 @@ actually degrade performance on fast links or with small files, so use with care
 '''
 
 
-def main(args: List[str]) -> None:
+def main(args: list[str]) -> None:
     raise SystemExit('This should be run as kitten transfer')
 
 
 if __name__ == '__main__':
     main(sys.argv)
 elif __name__ == '__doc__':
+    from kitty.cli import CompletionSpec
     cd = sys.cli_docs  # type: ignore
     cd['usage'] = usage
     cd['options'] = option_text
     cd['help_text'] = help_text
-    cd['short_desc'] = help_text
+    cd['short_desc'] = 'Transfer files easily over the TTY device'
+    cd['args_completion'] = CompletionSpec.from_string('type:file group:"Files"')
