@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
+from collections.abc import Callable, Iterable, Iterator
 from contextlib import suppress
 from dataclasses import dataclass, field
 from io import BytesIO
-from typing import TYPE_CHECKING, Any, Callable, Dict, FrozenSet, Iterable, Iterator, List, NoReturn, Optional, Set, Tuple, Type, Union, cast
+from typing import TYPE_CHECKING, Any, NoReturn, Optional, Union, cast
 
 from kitty.cli import CompletionSpec, get_defaults_from_seq, parse_args, parse_option_spec
 from kitty.cli_stub import RCOptions as R
-from kitty.constants import appname, list_kitty_resources, running_in_kitty
+from kitty.constants import list_kitty_resources, running_in_kitty
 from kitty.types import AsyncResponse
 
 if TYPE_CHECKING:
@@ -19,7 +20,7 @@ if TYPE_CHECKING:
     Boss = B
     Tab = T
 else:
-    Boss = Window = Tab = None
+    Boss = Window = Tab = object
 RCOptions = R
 
 
@@ -61,11 +62,11 @@ class StreamError(ValueError):
 
 class PayloadGetter:
 
-    def __init__(self, cmd: 'RemoteCommand', payload: Dict[str, Any]):
+    def __init__(self, cmd: 'RemoteCommand', payload: dict[str, Any]):
         self.payload = payload
         self.cmd = cmd
 
-    def __call__(self, key: str, opt_name: Optional[str] = None, missing: Any = None) -> Any:
+    def __call__(self, key: str, opt_name: str | None = None, missing: Any = None) -> Any:
         ans = self.payload.get(key, payload_get)
         if ans is not payload_get:
             return ans
@@ -75,13 +76,14 @@ class PayloadGetter:
 no_response = NoResponse()
 payload_get = object()
 ResponseType = Union[bool, str, None, NoResponse, AsyncResponse]
-CmdReturnType = Union[Dict[str, Any], List[Any], Tuple[Any, ...], str, int, float, bool]
+CmdReturnType = Union[dict[str, Any], list[Any], tuple[Any, ...], str, int, float, bool]
 CmdGenerator = Iterator[CmdReturnType]
 PayloadType = Optional[Union[CmdReturnType, CmdGenerator]]
 PayloadGetType = PayloadGetter
-ArgsType = List[str]
+ArgsType = list[str]
 ImageCompletion = CompletionSpec.from_string('type:file group:"Images"')
 ImageCompletion.extensions = 'png', 'jpg', 'jpeg', 'webp', 'gif', 'bmp', 'tiff'
+SUPPORTED_IMAGE_FORMATS = tuple(x.upper() for x in ImageCompletion.extensions if x != 'jpg')
 
 
 MATCH_WINDOW_OPTION = '''\
@@ -171,7 +173,7 @@ class ParsingOfArgsFailed(ValueError):
 
 class AsyncResponder:
 
-    def __init__(self, payload_get: PayloadGetType, window: Optional[Window]) -> None:
+    def __init__(self, payload_get: PayloadGetType, window: Window | None) -> None:
         self.async_id: str = payload_get('async_id', missing='')
         self.peer_id: int = payload_get('peer_id', missing=0)
         self.window_id: int = getattr(window, 'id', 0)
@@ -189,17 +191,17 @@ class AsyncResponder:
 class ArgsHandling:
 
     json_field: str = ''
-    count: Optional[int] = None
+    count: int | None = None
     spec: str = ''
     completion: CompletionSpec = field(default_factory=CompletionSpec)
-    value_if_unspecified: Tuple[str, ...] = ()
+    value_if_unspecified: tuple[str, ...] = ()
     minimum_count: int = -1
-    first_rest: Optional[Tuple[str, str]] = None
+    first_rest: tuple[str, str] | None = None
     special_parse: str = ''
-    args_choices: Optional[Callable[[], Iterable[str]]] = None
+    args_choices: Callable[[], Iterable[str]] | None = None
 
     @property
-    def args_count(self) -> Optional[int]:
+    def args_count(self) -> int | None:
         if not self.spec:
             return 0
         return self.count
@@ -211,7 +213,7 @@ class ArgsHandling:
         if self.completion:
             yield from self.completion.as_go_code(go_name + '.ArgCompleter', ' = ')
 
-    def as_go_code(self, cmd_name: str, field_types: Dict[str, str], handled_fields: Set[str]) -> Iterator[str]:
+    def as_go_code(self, cmd_name: str, field_types: dict[str, str], handled_fields: set[str]) -> Iterator[str]:
         c = self.args_count
         if c == 0:
             yield f'if len(args) != 0 {{ return fmt.Errorf("%s", "Unknown extra argument(s) supplied to {cmd_name}") }}'
@@ -251,7 +253,10 @@ class ArgsHandling:
                 elif self.special_parse.startswith('+'):
                     fields, sp = self.special_parse[1:].split(':', 1)
                     handled_fields.update(set(fields.split(',')))
-                    yield f'err = {sp}'
+                    if sp.startswith('!'):
+                        yield f'io_data.multiple_payload_generator, err = {sp[1:]}'
+                    else:
+                        yield f'err = {sp}'
                 else:
                     yield f'{dest}, err = {self.special_parse}'
                 yield 'if err != nil { return err }'
@@ -267,7 +272,7 @@ class ArgsHandling:
                 return
             if jt.startswith('choices.'):
                 yield f'if len(args) != 1 {{ return fmt.Errorf("%s", "Must specify exactly 1 argument for {cmd_name}") }}'
-                choices = ", ".join((f'"{x}"' for x in jt.split('.')[1:]))
+                choices = ", ".join(f'"{x}"' for x in jt.split('.')[1:])
                 yield 'switch(args[0]) {'
                 yield f'case {choices}:\n\t{dest} = args[0]'
                 yield f'default: return fmt.Errorf("%s is not a valid choice. Allowed values: %s", args[0], `{choices}`)'
@@ -283,9 +288,9 @@ class StreamInFlight:
 
     def __init__(self) -> None:
         self.stream_id = ''
-        self.tempfile: Optional[BytesIO] = None
+        self.tempfile: BytesIO | None = None
 
-    def handle_data(self, stream_id: str, data: bytes) -> Union[AsyncResponse, BytesIO]:
+    def handle_data(self, stream_id: str, data: bytes) -> AsyncResponse | BytesIO:
         from ..remote_control import close_active_stream
         def abort_stream() -> None:
             close_active_stream(self.stream_id)
@@ -321,15 +326,15 @@ class RemoteCommand:
     short_desc: str = ''
     desc: str = ''
     args: ArgsHandling = ArgsHandling()
-    options_spec: Optional[str] = None
+    options_spec: str | None = None
     response_timeout: float = 10.  # seconds
     string_return_is_error: bool = False
-    defaults: Optional[Dict[str, Any]] = None
+    defaults: dict[str, Any] | None = None
     is_asynchronous: bool = False
-    options_class: Type[RCOptions] = RCOptions
+    options_class: type[RCOptions] = RCOptions
     protocol_spec: str = ''
     argspec = args_count = args_completion = ArgsHandling()
-    field_to_option_map: Optional[Dict[str, str]] = None
+    field_to_option_map: dict[str, str] | None = None
     reads_streaming_data: bool = False
     disallow_responses: bool = False
 
@@ -350,7 +355,7 @@ class RemoteCommand:
             return self.defaults.get(name, missing)
         return missing
 
-    def windows_for_match_payload(self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType) -> List['Window']:
+    def windows_for_match_payload(self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType) -> list['Window']:
         if payload_get('all'):
             windows = list(boss.all_windows)
         else:
@@ -366,7 +371,7 @@ class RemoteCommand:
                     raise MatchError(payload_get('match'))
         return windows
 
-    def tabs_for_match_payload(self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType) -> List['Tab']:
+    def tabs_for_match_payload(self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType) -> list['Tab']:
         if payload_get('all'):
             return list(boss.all_tabs)
         match = payload_get('match')
@@ -387,14 +392,14 @@ class RemoteCommand:
     def windows_for_payload(
         self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType,
         window_match_name: str = 'match_window', tab_match_name: str = 'match_tab',
-    ) -> List['Window']:
+    ) -> list['Window']:
         if payload_get('all'):
             windows = list(boss.all_windows)
         else:
             window = window or boss.active_window
             windows = [window] if window else []
             if payload_get(window_match_name):
-                windows = list(boss.match_windows(payload_get(window_match_name)))
+                windows = list(boss.match_windows(payload_get(window_match_name), window))
                 if not windows:
                     raise MatchError(payload_get(window_match_name))
             if payload_get(tab_match_name):
@@ -406,7 +411,7 @@ class RemoteCommand:
                     windows += list(tab)
         return windows
 
-    def create_async_responder(self, payload_get: PayloadGetType, window: Optional[Window]) -> AsyncResponder:
+    def create_async_responder(self, payload_get: PayloadGetType, window: Window | None) -> AsyncResponder:
         return AsyncResponder(payload_get, window)
 
     def message_to_kitty(self, global_opts: RCOptions, opts: Any, args: ArgsType) -> PayloadType:
@@ -418,18 +423,18 @@ class RemoteCommand:
     def cancel_async_request(self, boss: 'Boss', window: Optional['Window'], payload_get: PayloadGetType) -> None:
         pass
 
-    def handle_streamed_data(self, data: bytes, payload_get: PayloadGetType) -> Union[BytesIO, AsyncResponse]:
+    def handle_streamed_data(self, data: bytes, payload_get: PayloadGetType) -> BytesIO | AsyncResponse:
         stream_id = payload_get('stream_id')
         if not stream_id or not isinstance(stream_id, str):
             raise StreamError('No stream_id in rc payload')
         return self.stream_in_flight.handle_data(stream_id, data)
 
 
-def cli_params_for(command: RemoteCommand) -> Tuple[Callable[[], str], str, str, str]:
-    return (command.options_spec or '\n').format, command.args.spec, command.desc, f'{appname} @ {command.name}'
+def cli_params_for(command: RemoteCommand) -> tuple[Callable[[], str], str, str, str]:
+    return (command.options_spec or '\n').format, command.args.spec, command.desc, f'kitten @ {command.name}'
 
 
-def parse_subcommand_cli(command: RemoteCommand, args: ArgsType) -> Tuple[Any, ArgsType]:
+def parse_subcommand_cli(command: RemoteCommand, args: ArgsType) -> tuple[Any, ArgsType]:
     opts, items = parse_args(args[1:], *cli_params_for(command), result_class=command.options_class)
     if command.args.args_count is not None and command.args.args_count != len(items):
         if command.args.args_count == 0:
@@ -453,7 +458,7 @@ def command_for_name(cmd_name: str) -> RemoteCommand:
     return cast(RemoteCommand, getattr(m, cmd_name))
 
 
-def all_command_names() -> FrozenSet[str]:
+def all_command_names() -> frozenset[str]:
 
     def ok(name: str) -> bool:
         root, _, ext = name.rpartition('.')
